@@ -32,7 +32,9 @@ open Common
 let progressbar_init = Util.Progress.create "Depsolver_int.init_solver"
 let progressbar_univcheck = Util.Progress.create "Depsolver_int.univcheck"
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 module R = struct type reason = Diagnostic_int.reason end
 module S = EdosSolver.M(R)
@@ -113,26 +115,30 @@ let init_pool_univ ~global_constraints univ =
      * However, since they are global, I've to add the at the end, after
      * I have analyzed all packages in the universe. *)
     Array.init size (fun uid ->
-      if uid = globalid then ([],[]) else (* the last index *)
-        let pkg = Cudf.package_by_uid univ uid in
-        let dll = 
-          List.map (fun vpkgs ->
-            (vpkgs, CudfAdd.resolve_vpkgs_int univ vpkgs)
-          ) pkg.Cudf.depends 
-        in
-        let cl = 
-          List.map (fun vpkg ->
-            debug "Conflict %s" (Cudf_types_pp.string_of_vpkg vpkg);
-            (vpkg, CudfAdd.resolve_vpkg_int univ vpkg)
-          ) pkg.Cudf.conflicts
-        in
-        if pkg.Cudf.keep = `Keep_package then
-          CudfAdd.add_to_package_list keep (pkg.Cudf.package,None) uid;
-          (*
-        if pkg.Cudf.keep = `Keep_version then
-          CudfAdd.add_to_package_list keep (pkg.Cudf.package,Some (`Eq pkg.Cudf.version)) uid;
-          *)
-        (dll,cl)
+      try
+        if uid = globalid then ([],[]) else begin (* the last index *)
+          let pkg = Cudf.package_by_uid univ uid in
+          let dll = 
+            List.map (fun vpkgs ->
+              (vpkgs, CudfAdd.resolve_vpkgs_int univ vpkgs)
+            ) pkg.Cudf.depends 
+          in
+          let cl = 
+            List.map (fun vpkg ->
+              debug "Conflict %s" (Cudf_types_pp.string_of_vpkg vpkg);
+              (vpkg, CudfAdd.resolve_vpkg_int univ vpkg)
+            ) pkg.Cudf.conflicts
+          in
+          if pkg.Cudf.keep = `Keep_package then
+            CudfAdd.add_to_package_list keep (pkg.Cudf.package,None) uid;
+            (*
+          if pkg.Cudf.keep = `Keep_version then
+            CudfAdd.add_to_package_list keep (pkg.Cudf.package,Some (`Eq pkg.Cudf.version)) uid;
+            *)
+          (dll,cl)
+        end
+      with Not_found ->
+        fatal "Package uid not found during solver pool initialization. Packages uid must have no gaps in the given universe"
     )
   in
   if global_constraints then begin
@@ -198,142 +204,8 @@ let init_solver_pool map pool closure =
   SolverPool solverpool
 ;;
 
-(** pseudo boolean optimization *) 
-
-let newpred constraints id vp =
-  let neg_lit_vp = List.map (fun i -> S.lit_of_var i false) vp in
-  let pos_lit_vp = List.map (fun i -> S.lit_of_var i true) vp in
-  let neg_lit_id = S.lit_of_var id false in
-  let pos_lit_id = S.lit_of_var id true in
-  S.add_rule constraints (Array.of_list (neg_lit_id::pos_lit_vp)) [];
-  List.iter (fun lit ->
-    S.add_rule constraints [|lit;pos_lit_id|] []
-  ) neg_lit_vp
-;;
-
-let rempred constraints id vp =
-  let neg_lit_vp = List.map (fun i -> S.lit_of_var i false) vp in
-  let pos_lit_vp = List.map (fun i -> S.lit_of_var i true) vp in
-  let neg_lit_id = S.lit_of_var id false in
-  let pos_lit_id = S.lit_of_var id true in
-  S.add_rule constraints (Array.of_list (neg_lit_id::neg_lit_vp)) [];
-  List.iter (fun lit ->
-    S.add_rule constraints [|lit;pos_lit_id|] []
-  ) pos_lit_vp
-;;
-
-let changepred constraints id ivp uvp =
-  let pos_lit_ivp = List.map (fun i -> S.lit_of_var i true) ivp in
-  let neg_lit_uvp = List.map (fun i -> S.lit_of_var i false) uvp in
-  let neg_lit_id = S.lit_of_var id false in
-  let pos_lit_id = S.lit_of_var id true in
-  List.iter (fun lit ->
-    S.add_rule constraints [|lit;neg_lit_id|] []
-  ) pos_lit_ivp;
-  List.iter (fun lit ->
-    S.add_rule constraints [|lit;pos_lit_id|] []
-  ) neg_lit_uvp
-;;
-
-type domain =
-  |New
-  |Removed
-  |Solution
-  |Changed 
-and criteria =
-  |Count of domain
-  |NotUpToDate of domain
-  |UnsatRecommends of domain
-
-let init_criteria ?(closure=[]) criteria univ =
-  let module ISet = Set.Make(struct type t = int let compare = compare end) in
-  let to_set l = List.fold_right ISet.add l ISet.empty in
-  let tmppool = Array.init (Array.length criteria) (fun _ -> RefList.empty ()) in
-  let closure = to_set closure in
-  Cudf.iter_packages_by_name (fun name packages -> 
-    if name = "dose-dummy-request" then () else begin
-      let pkgidset = (to_set (List.map (Cudf.uid_by_package univ) packages)) in
-      let vpset = ISet.inter pkgidset closure in
-      let vpidl = ISet.elements vpset in
-      (*
-      debug "iter init_criteria : %s : %d - closure %d" name (List.length vpidl) (ISet.cardinal closure);
-      *)
-      if ISet.is_empty closure || not (ISet.is_empty vpset) then begin
-        let ivp,uvp = List.partition (fun id -> (Cudf.package_by_uid univ id).Cudf.installed) vpidl in
-        Array.iteri (fun criteriaid -> function
-          |Count(Changed) -> (* sum over all package names *)
-              RefList.push tmppool.(criteriaid) (ivp,uvp)
-          |Count(Removed) when ivp <> [] -> (* at least one installed version *)
-              RefList.push tmppool.(criteriaid) (vpidl,[])
-          |Count(New) when ivp = [] -> (* no installed versions *)
-              RefList.push tmppool.(criteriaid) (vpidl,[])
-          |UnsatRecommends(Solution) -> ()
-          |NotUpToDate(Solution) -> ()
-          |_ -> ()
-        ) criteria
-      end
-    end;
-  ) univ;
-  let pbopool = ref [] in
-  Array.iteri (fun criteriaid t ->
-    let l = RefList.to_list tmppool.(criteriaid) in
-    let s = List.length l in
-    (*  debug "pull criteriaid %d - len %d" criteriaid s; *)
-    pbopool := (t,l,s,criteriaid) :: !pbopool
-  ) criteria;
-  !pbopool
-;;
-
-let init_solver_criteria map solver pbopool =
-  (*
-  debug "init_solver_criteria";
-  *)
-  List.iter (function
-    |Count(New),l,s,cid ->
-      let nameid = ref 0 in
-      List.iter (fun (vpidl,_) ->
-        let vpid = (List.map map#vartoint vpidl) in
-        (*
-        debug "new crit : %d - cid : %d - nameid : %d - versions : %s" !nameid cid (S.pboidx solver cid !nameid) (String.concat "," (List.map string_of_int vpid)) ;
-        *)
-        newpred solver (S.pboidx solver cid !nameid) vpid;
-        incr nameid;
-      ) l
-    
-    |Count(Removed),l,s,cid ->
-      let nameid = ref 0 in
-      List.iter (fun (vpidl,_) ->
-        let vpid = (List.map map#vartoint vpidl) in 
-        (*
-        debug "rem crit : %d - cid : %d - nameid : %d - versions : %s" !nameid cid (S.pboidx solver cid !nameid) (String.concat "," (List.map string_of_int vpid)) ;
-        *)
-        rempred solver (S.pboidx solver cid !nameid) vpid;
-        incr nameid;
-      ) l
-
-    |UnsatRecommends(Solution),l,s,cid -> ()
-    |NotUpToDate(Solution),l,s,cid -> ()
- 
-    |Count(Changed),l,s,cid -> 
-      let nameid = ref 0 in
-      List.iter (fun (l1,l2) ->
-        let ivp = List.map map#vartoint l1 in
-        let uvp = List.map map#vartoint l2 in 
-        (*
-        debug "chg crit : %d - cid : %d - nameid : %d - inst versions : %s - uninst versions : %s" 
-        !nameid cid (S.pboidx solver cid !nameid) (String.concat "," (List.map string_of_int ivp))
-        (String.concat "," (List.map string_of_int uvp)) ;
-        *)
-        changepred solver (S.pboidx solver cid !nameid) ivp uvp;
-        incr nameid;
-      ) l
-    |_ -> warning "Optimization criteria unsupported"
-  ) pbopool
-;;
-
-
 (** initalise the sat solver. operate only on solver ids *)
-let init_solver_cache ?(buffer=false) ?(pbopool=[]) varpool =
+let init_solver_cache ?(buffer=false) varpool =
   let varpool = strip_solver_pool varpool in
   let num_conflicts = ref 0 in
   let num_disjunctions = ref 0 in
@@ -385,13 +257,7 @@ let init_solver_cache ?(buffer=false) ?(pbopool=[]) varpool =
   in
 
   Util.Progress.set_total progressbar_init varsize ;
-  let pbo = Array.make (List.length pbopool) 0 in
-  List.iter (fun (_,_,x,cid) -> pbo.(cid) <- x) pbopool;
-  if pbopool <> [] then 
-    Array.iteri (fun criteriaid len ->
-      debug "init_solver_cache criteriaid : %d - pbovar : %d - nvar : %d" criteriaid len varsize;
-    ) pbo;
-  let constraints = S.initialize_problem ~buffer ~pbo varsize in
+  let constraints = S.initialize_problem ~buffer varsize in
 
   Array.iteri (fun id (dll,cl) ->
     Util.Progress.progress progressbar_init;
@@ -417,86 +283,42 @@ let solve ?tested solver request =
   S.reset solver.constraints;
 
   let result solve collect var =
-    if solve solver.constraints var then
-      let get_assignent ?(all=true) () =
-        List.map (fun i -> 
-          if not(Option.is_none tested) then
-            (Option.get tested).(i) <- true;
-          solver.map#inttovar i
-        )(S.assignment_true solver.constraints)
-      in
+    (* Real call to the SAT solver *)
+    if solve solver.constraints var then begin
+      let l = S.assignment_true solver.constraints in
+      List.iter (fun i -> 
+        if not(Option.is_none tested) then (
+          (Option.get tested).(i) <- true;
+        )
+      ) l;
+      let get_assignent ?(all=true) () = List.map solver.map#inttovar l in
       Diagnostic_int.Success(get_assignent)
-    else
+    end else
       let get_reasons () = collect solver.constraints var in
       Diagnostic_int.Failure(get_reasons)
   in
 
   match request with
-  |Diagnostic_int.Sng (None,i) ->
+  |(None,[i]) ->
       result S.solve S.collect_reasons (solver.map#vartoint i)
-  |Diagnostic_int.Lst (None,il) ->
+  |(None,il) ->
       result S.solve_lst S.collect_reasons_lst (List.map solver.map#vartoint il)
 
-  |Diagnostic_int.Sng (Some k,i) ->
+  |(Some k,[i]) ->
       result S.solve_lst S.collect_reasons_lst (List.map solver.map#vartoint [k;i])
-  |Diagnostic_int.Lst (Some k,il) ->
+  |(Some k,il) ->
       result S.solve_lst S.collect_reasons_lst (List.map solver.map#vartoint (k::il))
 ;;
 
-(* callback : (int array, Diagnostic_int.result) -> unit *)
-let solve_pbo ?callback solver request =
-  S.reset solver.constraints;
-
-  let result solve collect var =
-    if solve solver.constraints var then
-      let get_assignent ?(all=true) () =
-        List.map (solver.map#inttovar)
-        (S.assignment_true solver.constraints)
-      in
-      Diagnostic_int.Success(get_assignent)
-    else
-      let get_reasons () = collect solver.constraints var in
-      Diagnostic_int.Failure(get_reasons)
-  in
-
-  let callback =
-    match callback with
-    |None -> fun _ -> ()
-    |Some f -> fun (criteria,state) ->
-      let get_assignent ?(all=true) () =
-        List.map (solver.map#inttovar)
-        (S.assignment_true state)
-      in
-      f (criteria,Diagnostic_int.Success(get_assignent))
-  in
-
-  match request with
-  |Diagnostic_int.Sng (None,i) ->
-      begin
-        let ss = S.solve_pbo callback in
-        result ss S.collect_reasons (solver.map#vartoint i)
-      end
-  |Diagnostic_int.Sng (Some k,i) ->
-      failwith "Not implemented yet"
-      (* result S.solve_lst S.collect_reasons_lst (List.map solver.map#vartoint [k;i]) *)
-  |_ -> failwith "Not implemented yet"
-;;
-
-
 (* this function is used to "distcheck" a list of packages *)
 let pkgcheck global_constraints callback solver tested id =
   (* global id is a fake package id encoding the global constraints of the
    * universe. it is the last element of the id array *)
-  let req = 
-    if global_constraints then begin
-      let globalid = (Array.length tested) - 1 in
-      Diagnostic_int.Sng (Some globalid,id) 
-    end else
-      Diagnostic_int.Sng (None,id)
-  in
+  let globalid = (Array.length tested) - 1 in
+  let req = if global_constraints then begin (Some globalid,[id]) end else (None,[id]) in
   let res =
     Util.Progress.progress progressbar_univcheck;
-    if not(tested.(id)) then 
+    if not(tested.(id)) then
       solve ~tested solver req 
     else begin
       (* this branch is true only if the package was previously
@@ -529,7 +351,6 @@ let pkgcheck global_constraints callback solver tested id =
 let init_solver_univ ?(global_constraints=true) ?(buffer=false) univ =
   let map = new identity in
   let cudfpool = init_pool_univ global_constraints univ in
-  (* let globalid = (Array.length (strip_cudf_pool cudfpool)) - 1 in *)
   let varpool = SolverPool (strip_cudf_pool cudfpool) in
   let constraints = init_solver_cache ~buffer varpool in
   let solver = { constraints = constraints ; map = map } in
@@ -543,13 +364,12 @@ let init_solver_univ ?(global_constraints=true) ?(buffer=false) univ =
     @param closure subset of packages used to initialize the solver
 *)
 (* pool = cudf pool - closure = dependency clousure . cudf uid list *)
-let init_solver_closure ?(buffer=false) ?(pbopool=[]) cudfpool closure =
+let init_solver_closure ?(buffer=false) cudfpool closure =
   let map = new intprojection (List.length closure) in
   List.iter map#add closure;
   (* let globalid = (Array.length (strip_cudf_pool cudfpool)) - 1 in *)
   let varpool = init_solver_pool map cudfpool closure in
-  let constraints = init_solver_cache ~buffer ~pbopool varpool in
-  init_solver_criteria map constraints pbopool;
+  let constraints = init_solver_cache ~buffer varpool in
   let solver = { constraints = constraints ; map = map } in
   solver
 ;;
@@ -613,11 +433,12 @@ let dependency_closure_cache ?(maxdepth=max_int) ?(conjunctive=false) pool idlis
     @param universe the package universe
     @param pkglist a subset of [universe]
 *)
-let dependency_closure ?(maxdepth=max_int) ?(conjunctive=false) universe pkglist =
-  let pool = init_pool_univ ~global_constraints:false universe in
+let dependency_closure ?(maxdepth=max_int) ?(conjunctive=false) ?(global_constraints=false) universe pkglist =
+  let pool = init_pool_univ ~global_constraints universe in
+  let globalid = Cudf.universe_size universe in
   let idlist = List.map (CudfAdd.vartoint universe) pkglist in
-  let l = dependency_closure_cache ~maxdepth ~conjunctive pool idlist in
-  List.map (CudfAdd.inttovar universe) l
+  let l = dependency_closure_cache ~maxdepth ~conjunctive pool (globalid::idlist) in
+  List.filter_map (fun p -> if p <> globalid then Some (CudfAdd.inttovar universe p) else None) l
 ;;
 
 (*    XXX : elements in idlist should be included only if because
