@@ -14,7 +14,9 @@ open Common
 open CudfAdd
 open Defaultgraphs
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 module ICG = Strongconflicts_int.CG
 
