@@ -10,19 +10,21 @@
 (*  library, see the COPYING file for more information.                               *)
 (**************************************************************************************)
 
-
+module Pcre = Re_pcre
 (* Remember the original hashtable module from Ocaml standard library,
     whose name will be overriden by opening Extlib. *)
 module OCAMLHashtbl = Hashtbl
 
 open ExtLib
 
-(* Include internal debugging functions for this module (debug, info, warning, fatal). *)
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 let equal = Cudf.(=%)
 let compare = Cudf.(<%)
 
+let sort = List.sort ~cmp:compare
 let hash p = Hashtbl.hash (p.Cudf.package,p.Cudf.version)
 
 module Cudf_hashtbl =
@@ -40,8 +42,7 @@ module Cudf_set =
 
 let to_set l = List.fold_right Cudf_set.add l Cudf_set.empty
 
-
-(* encode - decode *)
+(** Encode - Decode *)
 
 (* Specialized hashtable for encoding strings efficiently. *)
 module EncodingHashtable =
@@ -59,7 +60,6 @@ module DecodingHashtable =
     let hash = (fun s -> (Char.code s.[1]) * 1000 + (Char.code s.[2]) )
   end)
 
-
 (*  "hex_char char" returns the ASCII code of the given character
     in the hexadecimal form, prefixed with the '%' sign.
     e.g. hex_char '+' = "%2b" *)
@@ -75,31 +75,15 @@ let hex_char char = Printf.sprintf "%%%02x" (Char.code char);;
     e.g. DecodingHashtable.find dec_ht "%2b" = "+" 
 *)
 let init_hashtables enc_ht dec_ht =
-
-  (* "all_ascii_chars" is a list containing characters with 
-     all possible ASCII codes (i.e. between 0 and 255). *)
-  let all_ascii_chars = 
-    (*  "all_numbers_from_to first last" generates a list of
-	all integers between the first one and the last one
-	(including both ends). *)
-    let rec all_numbers_from_to first last =
-      if first <= last
-      then first :: (all_numbers_from_to (first + 1) last)
-      else []
-    in
-    (* Convert each number from 0 to 255 to a character with the given ASCII code. *)
-    List.map Char.chr (all_numbers_from_to 0 255)
-      
-  in
-  (* One by one add all the possible pairs (char, hexed_char)
-      to both hashtables. *)
-  List.iter (fun char -> 
-    let char_as_string = String.make 1 char in
-    let hexed_char = hex_char char in
-    EncodingHashtable.add enc_ht char_as_string hexed_char;
-    DecodingHashtable.add dec_ht hexed_char char_as_string
-      ) all_ascii_chars
-    ;;
+  let n = ref 255 in
+  while !n >= 0 do
+    let schr = String.make 1 (Char.chr !n) in
+    let hchr = Printf.sprintf "%%%02x" !n in
+    EncodingHashtable.add enc_ht schr hchr;
+    DecodingHashtable.add dec_ht hchr schr;
+    decr n;
+  done
+;;
 
 (* Create and initialize twin hashtables,
    one for encoding and one for decoding. *)
@@ -108,7 +92,7 @@ let dec_ht = DecodingHashtable.create 256;;
 init_hashtables enc_ht dec_ht;;
 
 (* encode *)
-let encode_single s   = EncodingHashtable.find enc_ht s;;
+let encode_single s = EncodingHashtable.find enc_ht s;;
 let not_allowed_regexp = Pcre.regexp "[^a-zA-Z0-9@/+().-]";;
 
 let encode s =
@@ -123,7 +107,7 @@ let decode s =
   Pcre.substitute ~rex:encoded_char_regexp ~subst:decode_single s
 ;;
 
-(* formatting *)
+(** Pretty Printing *)
 
 let string_of pp arg =
   ignore(pp Format.str_formatter arg);
@@ -139,21 +123,101 @@ let pp_package fmt pkg =
 let string_of_version = string_of pp_version
 let string_of_package = string_of pp_package
 
+type pp = Cudf.package -> string * string * (string * (string * bool)) list
+
+(** [default_pp] default package printer. If the version of the package is
+  * a negative number, the version version if printed as "nan" *)
+let default_pp pkg =
+  let v = if pkg.Cudf.version > 0 then string_of_version pkg else "nan" in
+  (pkg.Cudf.package,v,[])
+;;
+
+let pp from_cudf ?(fields=[]) ?(decode=decode) pkg =
+    let (p,i) = (pkg.Cudf.package,pkg.Cudf.version) in
+    let v = if i > 0 then snd(from_cudf (p,i)) else "nan" in
+    let default_fields = ["architecture";"source";"sourcenumber";"essential"] in
+    let f b l =
+      List.filter_map (fun k ->
+        try Some(k,(decode(Cudf.lookup_package_property pkg k),b))
+        with Not_found -> None
+      ) l
+    in
+    let l = (f false fields)@(f true default_fields) in
+    (decode p,decode v,l)
+;;
+
+let pp_vpkg pp fmt vpkg =
+  let string_of_relop = function
+      `Eq -> "="
+    | `Neq -> "!="
+    | `Geq -> ">="
+    | `Gt -> ">"
+    | `Leq -> "<="
+    | `Lt -> "<"
+  in
+  match vpkg with
+  |(p,None) ->
+      let (p,_,_) = pp {Cudf.default_package with Cudf.package = p} in
+      Format.fprintf fmt "%s" p
+  |(p,Some(c,v)) ->
+      debug "pp_vpkg %s %s %i" p (string_of_relop c) v;
+      let (p,v,_) = pp {Cudf.default_package with Cudf.package = p ; version = v} in
+      Format.fprintf fmt "%s (%s %s)" p (string_of_relop c) v
+;;
+
+let pp_vpkglist pp fmt =
+  let pp_list fmt ~pp_item ~sep l =
+    let rec aux fmt = function
+      | [] -> assert false
+      | [last] -> (* last item, no trailing sep *)
+          Format.fprintf fmt "@,%a" pp_item last
+      | vpkg :: tl -> (* at least one package in tl *)
+          Format.fprintf fmt "@,%a%s" pp_item vpkg sep ;
+          aux fmt tl
+    in
+    match l with
+    | [] -> ()
+    | [sole] -> pp_item fmt sole
+    | _ -> Format.fprintf fmt "@[<h>%a@]" aux l
+  in
+  pp_list fmt ~pp_item:(pp_vpkg pp) ~sep:" | "
+;;
+
+
 module StringSet = Set.Make(String)
 
+let add_to_package_list h n p =
+  try let l = Hashtbl.find h n in l := p :: !l
+  with Not_found -> Hashtbl.add h n (ref [p])
+
+let get_package_list h n = try !(Hashtbl.find h n) with Not_found -> []
+
 let pkgnames universe =
   Cudf.fold_packages (fun names pkg ->
     StringSet.add pkg.Cudf.package names
   ) StringSet.empty universe
 
+let pkgnames_ universe =
+  let h = Hashtbl.create (Cudf.universe_size universe) in
+  Cudf.iter_packages (fun pkg ->
+    add_to_package_list h pkg.Cudf.package pkg
+  ) universe
 
 let add_properties preamble l =
   List.fold_left (fun pre prop ->
     {pre with Cudf.property = prop :: pre.Cudf.property }
   ) preamble l
 
+let get_property prop pkg =
+   try Cudf.lookup_package_property pkg prop
+   with Not_found -> begin
+     warning "%s missing" prop;
+     raise Not_found
+   end
+;;
+
 let is_essential pkg =
-  try (Cudf.lookup_package_property pkg "essential") = "yes"
+  try Cudf.lookup_package_property pkg "essential" = "true"
   with Not_found -> false
 
 let realversionmap pkglist =
@@ -163,25 +227,30 @@ let realversionmap pkglist =
   ) pkglist ;
   h
 
-let vartoint = Cudf.uid_by_package 
+let vartoint universe p =
+  try Cudf.uid_by_package universe p
+  with Not_found-> begin
+    warning 
+    "package %s is not associate with an integer in the given universe"
+    (string_of_package p);
+    raise Not_found
+  end
+
 let inttovar = Cudf.package_by_uid
 
-let add_to_package_list h n p =
-  try let l = Hashtbl.find h n in l := p :: !l
-  with Not_found -> Hashtbl.add h n (ref [p])
-
-let get_package_list h n = try !(Hashtbl.find h n) with Not_found -> []
-
-let unique l = 
-  List.rev (List.fold_left (fun results x -> 
-    if List.mem x results then results 
+let normalize_set (l : int list) = 
+  (* List.rev(Util.list_unique l) *)
+  List.rev (List.fold_left (fun results x ->
+    if List.mem x results then results
     else x::results) [] l
   )
-;;
+(*
+  let module Int = struct type t = int let compare = (-) end in
+  let module ISet = Set.Make(Int) in
+  ISet.elements (List.fold_left (fun acc x -> ISet.add x acc) ISet.empty l)
+*)
 
-let normalize_set (l : int list) = unique l
-
-(* (pkgname,constr) -> pkg *)
+(* vpkg -> pkg list *)
 let who_provides univ (pkgname,constr) = 
   let pkgl = Cudf.lookup_packages ~filter:constr univ pkgname in
   let prol = Cudf.who_provides ~installed:false univ (pkgname,constr) in
@@ -203,6 +272,8 @@ let resolve_deps univ vpkgs =
 let who_depends univ pkg = 
   List.map (resolve_deps univ) pkg.Cudf.depends
 
+type ctable = (int, int list ref) ExtLib.Hashtbl.t
+
 let who_conflicts conflicts_packages univ pkg = 
   if (Hashtbl.length conflicts_packages) = 0 then
     warning "Either there are no conflicting packages in the universe or you
@@ -235,7 +306,7 @@ let compute_pool universe =
   let c = Array.init size (fun i -> get_package_list conflicts i) in
   let d =
     Array.init size (fun i ->
-      let p = inttovar universe i in
+      let p = Cudf.package_by_uid universe i in
       List.map (resolve_vpkgs_int universe) p.Cudf.depends
     )
   in
@@ -248,9 +319,11 @@ let cudfop = function
   |Some("<=",v) -> Some(`Leq,v)
   |Some(">=",v) -> Some(`Geq,v)
   |Some("=",v) -> Some(`Eq,v)
+  |Some("!=",v) -> Some(`Neq,v)
   |Some("ALL",v) -> None
   |None -> None
   |Some(c,v) -> fatal "%s %s" c v
+;;
 
 let latest pkglist =
   let h = Hashtbl.create (List.length pkglist) in
@@ -264,14 +337,3 @@ let latest pkglist =
   ) pkglist;
   Hashtbl.fold (fun _ v acc -> v::acc) h []
 ;;
-
-let pp from_cudf ?(decode=decode) pkg =
-  let (p,i) = (pkg.Cudf.package,pkg.Cudf.version) in
-  let v = if i > 0 then snd(from_cudf (p,i)) else "nan" in
-  let l =
-    List.filter_map (fun k ->
-      try Some(k,decode(Cudf.lookup_package_property pkg k))
-      with Not_found -> None
-    ) ["architecture";"source";"sourcenumber";"essential"]
-  in (decode p,decode v,l)
-;;
