@@ -10,6 +10,8 @@
 (*  library, see the COPYING file for more information.                               *)
 (**************************************************************************************)
 
+module Pcre = Re_pcre
+
 open OUnit
 open Common
 
@@ -57,30 +59,6 @@ let test_cudf =
   )
 ;;
 
-let test_sqlite =
-  "sqlite" >:: (fun _ ->
-    let (protocol,(userOpt,passOpt,hostOpt,portOpt,path),queryOpt) =
-      Input.parse_uri "sqlite:///path/to/file" 
-    in assert_equal true (protocol = `Sqlite && path = "/path/to/file")
-  ) 
-;;
-
-let test_pgsql =
-  "pgsql" >:: (fun _ ->
-    let (protocol,(userOpt,passOpt,hostOpt,portOpt,path),queryOpt) =
-      Input.parse_uri "pgsql://test:tester@localhost:10/dbname?query=lalalal;v2=k2"
-    in assert_equal true (
-      protocol = `Pgsql && 
-      userOpt = Some "test" &&
-      passOpt = Some "tester" &&
-      hostOpt = Some "localhost" &&
-      portOpt = Some "10" &&
-      path = "dbname" &&
-      queryOpt = Some "lalalal;v2=k2"
-    )
-  ) 
-;;
-
 let parse_uri =
   "parse uri" >::: [
     test_deb_local;
@@ -88,8 +66,6 @@ let parse_uri =
     test_hdlist;
     test_synth;
     test_cudf;
-    test_sqlite;
-    (* test_pgsql; *)
   ]
 
 module S = CudfAdd.Cudf_set
@@ -139,10 +115,11 @@ let test_lookup_packages =
 
 let (test_encode, test_decode) =
   (* Some useful very long strings for testing encoding and decoding. *)
-  let a_lot_of =  (* a huge number *)
+(*  let a_lot_of =  (* a huge number *)
     (Pcre.config_match_limit + 111)
   in
-  let a_lot_of_a           = String.make a_lot_of 'a'
+  *)
+  let a_lot_of_a           = String.make 10000 'a'
   (* This test takes too much time... 
   and a_lot_of_pipes       = String.make a_lot_of '|' in
   let a_lot_of_hexed_pipes = ExtLib.String.replace_chars (fun _ -> "%7c") a_lot_of_pipes *)
@@ -208,12 +185,48 @@ let (test_encode, test_decode) =
 
 *)
 
+let test_shell_lexer =
+  let test_triplets = [
+    ("simple", "foo bar baz", ["foo"; "bar"; "baz"]);
+    ("simple quoted", "\"foo\" 'bar'", ["foo"; "bar"]);
+    ("simple escaped", "foo\\ bar foo\\\\bar foo\\\"bar foo\\'bar", ["foo bar"; "foo\\bar"; "foo\"bar"; "foo'bar"]);
+    ("composed", "foo\"bar\"'baz'", ["foobarbaz"]);
+    ("dquote escaped", "\"foo\\\"bar\"", ["foo\"bar"]);
+    ("uquote starts with escape", "\\\"foo \\'bar \\ baz \\\\blub", ["\"foo"; "'bar"; " baz"; "\\blub"]);
+    ("dquote starts with escape", "\"\\\"foo\" \"\\\\bar\"", ["\"foo"; "\\bar"]);
+    ("uquote ends with escape", "foo\\\" bar\\' baz\\  blub\\\\", ["foo\""; "bar'"; "baz "; "blub\\"]);
+    ("dquote ends with escape", "\"foo\\\"\" \"bar\\\\\"", ["foo\""; "bar\\"]);
+    ("compose uquote dquote", "foo\"bar\"baz", ["foobarbaz"]);
+    ("compose uquote squote", "foo'bar'baz", ["foobarbaz"]);
+    ("compose dquote squote", "\"foo\"'bar'\"baz\"", ["foobarbaz"]);
+    ("empty", "\"\" ''", [""; ""]);
+    ("compose empty", "\"\"''\"\"\"\"''''", [""]);
+    ("multispace", "foo    bar   \t   baz", ["foo"; "bar"; "baz"]);
+  ] in
+  "lexing tests" >::: (
+    List.map (fun (test_name, exec_string, argv) ->
+        ("shell_lexer " ^ test_name) >:: (fun _ ->
+            let l = Shell_lexer.parse_string exec_string in
+            assert_equal (List.length l) (List.length argv)
+              ~msg:(Printf.sprintf
+                      "Expected argument vector of length %d but got one of length %d"
+                      (List.length argv) (List.length l));
+            List.iter2 (assert_equal
+              ~msg:(Printf.sprintf
+                      "String to parse:\n\t%s\nExpected:\n\t%s\nBut got:\n\t%s\n"
+                      exec_string (String.concat "\n\t" argv)
+                      (String.concat "\n\t" l))) l argv
+          )
+      ) test_triplets
+  )
+
 let all = 
   "all tests" >::: [
     parse_uri ;
     test_encode;
     test_decode;
     test_lookup_packages;
+    test_shell_lexer;
   ]
 
 let main () =
