@@ -10,12 +10,11 @@
 (*  library, see the COPYING file for more information.                     *)
 (****************************************************************************)
 
-type debtypes = [ `Edsp | `Deb ]
+type debtypes = [ `Edsp | `Deb | `DebSrc ]
 type rpmtypes = [ `Synthesis | `Hdlist ]
-type osgitypes = [ `Eclipse ]
-type othertypes = [ `Csw ]
+type othertypes = [ `Pef | `Csw ]
 
-type filetypes = [ `Cudf | debtypes | rpmtypes | osgitypes | othertypes ]
+type filetypes = [ `Cudf | debtypes | rpmtypes | othertypes ]
 
 val supported_input_types : filetypes list
 
