@@ -17,8 +17,12 @@ module SSet = Set.Make(String)
 open ExtLib
 open Common
 open Packages
+module Version = Versioning.Debian
+
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
-include Util.Logging(struct let label = __FILE__ end) ;;
 module SMap = Map.Make (String)
 
 type tables = {
@@ -52,6 +56,7 @@ type options = {
   foreign : string list ; (* list of foreign architectures *)
   host : string option;   (* the host architecture - cross compile *)
   ignore_essential : bool;
+  builds_from : bool;
 }
 
 let default_options = {
@@ -59,28 +64,28 @@ let default_options = {
   native = None;
   foreign = [];
   host = None;
-  ignore_essential = false
+  ignore_essential = false;
+  builds_from = false;
 }
 
-let add_name_arch n a = CudfAdd.encode (Printf.sprintf "%s:%s" a n)
-
-(* add arch info to a vpkg 
- - if it's a :any dependency then just encode the name without arch information :
-   means that this dependency/conflict can be satified by any packages
- - if it is a package:arch dependency, then encode it as such
- - if the package is architecture all, then all dependencies are interpreted as
-   dependencies on native architecture packages.
- - otherwise all dependencies are satisfied by packages of the same architecture
-   of the package we are considering *)
-(* XXX we should use one regexp and check there results instead of three
- * different String.* functions *)
-let add_arch native_arch package_arch name = 
-    if package_arch = "all" then 
-      add_name_arch name native_arch
-    else
-      add_name_arch name package_arch
+let add_name_arch n a = CudfAdd.encode (Printf.sprintf "%s:%s" n a)
+let add_arch native_arch name = function
+  |"all" -> add_name_arch name native_arch
+  |package_arch -> add_name_arch name package_arch
 ;;
 
+(** add arch info to a vpkg 
+ - if it's a :any dependency then just encode the name without arch information :
+   means that this dependency/conflict can be satified by any packages
+ - if it is a :native dependency then add the native architecture
+ - if it is a package:arch dependency, then encode it as such
+ - if the package dependency does not have any annotations :
+   + if the package is architecture all, then all dependencies are interpreted as
+     dependencies on native architecture packages.
+   + otherwise all dependencies are satisfied by packages of the same architecture
+     of the package we are considering 
+Pcre.regexp "^(.* )(:(any|native|.* )){1}$" ;;
+*)
 let add_arch_deps native_arch package_arch name = 
   try 
     match String.split name ":" with
@@ -88,24 +93,14 @@ let add_arch_deps native_arch package_arch name =
     |n,"native" -> add_name_arch n native_arch
     |n,a -> add_name_arch n a
   with ExtString.Invalid_string ->
-    add_arch native_arch package_arch name
+    add_arch native_arch name package_arch
 ;;
-(*
-  function
-  |name when String.ends_with name ":any" -> (CudfAdd.encode name)
-  |name when String.ends_with name ":native" -> add_name_arch (String.slice ~last:(-7) name) native_arch
-  |name when String.contains name ':' -> 
-      let name, arch = String.split name ":" in
-      add_name_arch name arch
-  |name when package_arch = "all" -> add_name_arch name native_arch
-  |name -> add_name_arch name package_arch
-*)
 
 let add_arch_l ?(deps=false) native_arch package_arch l = 
   if deps then
     List.map (fun (n,c) -> ((add_arch_deps native_arch package_arch n),c)) l
   else
-    List.map (fun (n,c) -> ((add_arch native_arch package_arch n),c)) l
+    List.map (fun (n,c) -> ((add_arch native_arch n package_arch),c)) l
 ;;
 
 let clear tables =
@@ -226,7 +221,9 @@ let get_real_version tables (name,cudfversion) =
   let package = 
     (* XXX this is a hack. I should record the name with the architecture *)
     let n = CudfAdd.decode name in
-    try let (n,a) = ExtString.String.split n ":" in if a <> "any" then a else n
+    try
+      let (n,a) = ExtString.String.split n ":" in
+      if n = "src" then a else n
     with Invalid_string -> n
   in
   try
@@ -302,7 +299,8 @@ let loadlp ?(enc=false) tables l =
 
 let preamble = 
   let l = [
-    (* number,type,architecture are mandatory properties -- no default *)
+    (* name,number,type,architecture are mandatory properties -- no default *)
+    ("name", (`String None));
     ("number",(`String None));
     ("type",(`String None));
     ("architecture",(`String None));
@@ -324,6 +322,11 @@ let preamble =
 
 let add_extra_default extras tables pkg =
   let check = function [] :: _ -> failwith (Printf.sprintf "Malformed dep (%s %s)" pkg.name pkg.version) |l -> l in
+  let name = if String.starts_with pkg.name "src:" then
+      String.sub pkg.name 4 ((String.length pkg.name) - 4)
+    else pkg.name
+  in
+  let name = ("name",`String name) in
   let number = ("number",`String pkg.version) in
   let architecture = ("architecture",`String pkg.architecture) in
   let priority = ("priority",`String pkg.priority) in
@@ -363,7 +366,7 @@ let add_extra_default extras tables pkg =
     |(_,`String "") -> None
     |e -> Some e
   )
-  [priority; architecture; number;
+  [priority; name; architecture; number;
   source; sourcenumber; sourceversion; 
   recommends; replaces;
   essential;build_essential]@ l
@@ -400,14 +403,14 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
       (* if the package is a source package the name does not need an
        * architecture annotation. Nobody depends on it *)
       if Sources.is_source pkg then encpkgname
-      else add_arch native_arch pkgarch pkg.name 
+      else add_arch native_arch pkg.name pkgarch
     in
     let _version = get_cudf_version tables (pkg.name,pkg.version)  in
     let _provides = 
       let archlessprovide = (encpkgname,None) in
       let multiarchprovides = 
         match pkg.multiarch with
-        |(`None|`Same) ->
+        |(`No|`Same) ->
            (* pkgarch provides *)
              (add_arch_l native_arch pkgarch (loadlp tables pkg.provides))
         |`Foreign ->
@@ -417,7 +420,7 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
                 |arch when arch = pkgarch ->
                     (add_arch_l native_arch arch (loadlp tables pkg.provides))
                 |arch ->
-                    (add_arch native_arch arch pkg.name,Some(`Eq,_version)) ::
+                    (add_arch native_arch pkg.name arch,Some(`Eq,_version)) ::
                       (add_arch_l native_arch arch (loadlp tables pkg.provides))
               )
         |`Allowed ->
@@ -428,7 +431,7 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
            let l = 
              bind (loadlp tables pkg.provides) (fun (name, c) ->
                [(CudfAdd.encode (name^":any"), c);
-               ((add_arch native_arch pkgarch name),c)]
+               ((add_arch native_arch name pkgarch),c)]
              )
            in any::l
       in archlessprovide :: multiarchprovides
@@ -436,10 +439,10 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
     let _conflicts = 
       let originalconflicts = pkg.breaks @ pkg.conflicts in
       (* self conflict *)
-      let sc = (add_arch native_arch pkgarch pkg.name,None) in
+      let sc = (add_arch native_arch pkg.name pkgarch,None) in
       let multiarchconstraints = 
         match pkg.multiarch with
-        |(`None|`Foreign|`Allowed) -> 
+        |(`No|`Foreign|`Allowed) -> 
             (* conflict with all other packages with differents archs *)
             let mac = (encpkgname,None) in
             [sc; mac] 
@@ -448,7 +451,7 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
             let masc =
               List.filter_map (function
                 |arch when arch = pkgarch -> None
-                |arch -> Some(add_arch native_arch arch pkg.name,Some(`Neq,_version))
+                |arch -> Some(add_arch native_arch pkg.name arch,Some(`Neq,_version))
               ) (native_arch::options.foreign)
             in
             sc :: masc 
@@ -460,7 +463,7 @@ let tocudf tables ?(options=default_options) ?(inst=false) pkg =
 	in
         let realpackage = Util.StringHashtbl.mem tables.unit_table in
         match pkg.multiarch with
-        |(`None|`Foreign|`Allowed) -> 
+        |(`No|`Foreign|`Allowed) -> 
             bind (native_arch::options.foreign) (fun arch ->
                 add_arch_l native_arch arch (loadl tables originalconflicts)
               )
@@ -563,7 +566,7 @@ let lltocudf = loadll
 let ltocudf = loadl
 
 let load_list ?options l =
-  let timer = Util.Timer.create "Debian.Debcudf.load_list" in
+  let timer = Util.Timer.create "Debian.ToCudf" in
   Util.Timer.start timer;
   let tables = init_tables l in
   let pkglist = List.map (tocudf tables ?options) l in
@@ -571,7 +574,7 @@ let load_list ?options l =
   Util.Timer.stop timer pkglist
 
 let load_universe ?options l =
-  let timer = Util.Timer.create "Debian.Debcudf.load_universe" in
+  let timer = Util.Timer.create "Debian.ToCudf" in
   let pkglist = load_list ?options l in
   Util.Timer.start timer;
   let univ = Cudf.load_universe pkglist in
