@@ -18,25 +18,26 @@ type tables
 type extramap = (string * (string * Cudf_types.typedecl1)) list
 type options = {
   extras_opt : extramap ;
-  native : string ;
-  foreign : string list ;
-  target : string ;
+  native : string option; (** the native architecture *)
+  foreign : string list ; (** list of foreign architectures *)
+  host : string option;   (** the host architecture - cross compile *)
   ignore_essential : bool ;
+  builds_from : bool ;    (** whether to add the builds-from relationship from binary to source packages *)
 }
 
 val default_options : options
 
 (** initialize the version conversion tables *)
-val init_tables : ?step:int -> ?versionlist:Format822.version list -> Packages.package list -> tables
+val init_tables : ?step:int -> ?versionlist:Packages_types.version list -> Packages.package list -> tables
 
 val clear : tables -> unit
 (** return the cudf version associated to a tuple (name,version). 
  * return Not_found if there is not package or cudf version associated
  * to the tuple (name,version) *)
-val get_cudf_version : tables -> Format822.name * Format822.version -> int
+val get_cudf_version : tables -> Packages_types.name * Packages_types.version -> int
 
 (** return the real version associated to a Cudf package *)
-val get_real_version : tables -> Cudf_types.pkgname * Cudf_types.version -> Format822.version
+val get_real_version : tables -> Cudf_types.pkgname * Cudf_types.version -> Packages_types.version
 
 (** [tocudf tbl p] 
     convert the a debian package representation to cudf.
@@ -50,10 +51,10 @@ val get_real_version : tables -> Cudf_types.pkgname * Cudf_types.version -> Form
 val tocudf : tables -> ?options:options -> ?inst:bool -> Packages.package -> Cudf.package
 
 (** convert a debian dependency list in a cudf constraints formula *)
-val lltocudf : ?enc:bool -> tables -> Format822.vpkg list list -> Cudf_types.vpkgformula
+val lltocudf : ?enc:bool -> tables -> Packages_types.vpkg list list -> Cudf_types.vpkgformula
 
 (** convert a debian conflict list in a cudf constraints list *)
-val ltocudf  : ?enc:bool -> tables -> Format822.vpkg list -> Cudf_types.vpkglist
+val ltocudf  : ?enc:bool -> tables -> Packages_types.vpkg list -> Cudf_types.vpkglist
 
 (** declare the Cudf preamble used by cudf. Namely, debcudf add :
     - a property named {b Number} of type string containing the original debian version
